#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.test import TestCase
from django.test.client import RequestFactory
from django.template import Context, RequestContext, Template

from addendum.models import Snippet, SnippetTranslation


class ModelTests(TestCase):

    def test_unicode(self):
        s = Snippet.objects.create(key="holá", text="World")
        self.assertEqual(s.__str__(), "holá")


class TagTests(TestCase):
    """
    Tests that the template tag renders the correct text
    """

    def setUp(self):
        self.plain_snippet = Snippet.objects.create(key="plain",
                text="Hello, humans")
        self.rich_snippet = Snippet.objects.create(key="rich",
                text="<h1>Hello, humans</h1>")
        self.template_snippet = Snippet.objects.create(key="django",
                text="{{ dog|upper }}")
        SnippetTranslation.objects.create(
            snippet=self.plain_snippet,
            language='es',
            text="Hola, humanos",
        )
        SnippetTranslation.objects.create(
            snippet=self.plain_snippet,
            language='en-au',
            text="G'day, humans",
        )

    def test_has_snippet(self):
        """Ensure that the saved snippet text is displayed"""
        t = Template("""{% load addendum_tags %}{% snippet 'plain' %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "Hello, humans")

    def test_no_snippet(self):
        """Ensure that the default text is displayed"""
        t = Template("""{% load addendum_tags %}{% snippet 'missing' %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "Hello world")

    def test_plain_text(self):
        """Ensure that content is not escaped"""
        t = Template("""{% load addendum_tags %}{% snippet 'rich' %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "&lt;h1&gt;Hello, humans&lt;/h1&gt;")

    def test_richtext(self):
        """Ensure that with richtext argument content is escaped"""
        t = Template("""{% load addendum_tags %}{% snippet 'rich' richtext=True %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "<h1>Hello, humans</h1>")

    def test_raw_template_text(self):
        """Ensure template code is not compiled by default"""
        t = Template("""{% load addendum_tags %}{% snippet 'django' %}Hello world{% endsnippet %}""")
        c = Context({'dog': 'woof'})
        result = t.render(c)
        self.assertEqual(result, "{{ dog|upper }}")

    def test_template_text(self):
        """Ensure template code is rendered with the template option"""
        t = Template("""{% load addendum_tags %}{% snippet 'django' template=True %}Hello world{% endsnippet %}""")
        c = Context({'dog': 'woof'})
        result = t.render(c)
        self.assertEqual(result, "WOOF")

    def test_safe_template_text(self):
        t = Template("""{% load addendum_tags %}{% snippet 'django' template=True %}Hello world{% endsnippet %}""")
        c = Context({'dog': '<h1>woof</h1>'})
        result = t.render(c)
        self.assertEqual(result, "&lt;H1&gt;WOOF&lt;/H1&gt;")

        t = Template("""{% load addendum_tags %}{% snippet 'django' template=True safe=True %}Hello world{% endsnippet %}""")
        c = Context({'dog': '<h1>woof</h1>'})
        result = t.render(c)
        self.assertEqual(result, "<H1>WOOF</H1>")

    def test_variable_key_name(self):
        """Ensure a variable can be passed for the snippet key"""
        t = Template("""{% load addendum_tags %}{% snippet snippetname %}Hello world{% endsnippet %}""")
        c = Context({'snippetname': 'plain'})
        result = t.render(c)
        self.assertEqual(result, "Hello, humans")

    def test_translate_basic(self):
        """Translate based on passed language code"""
        t = Template("""{% load addendum_tags %}{% snippet 'plain' language='es' %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "Hola, humanos")

    def test_missing_translation(self):
        """Return default language for missing translation"""
        t = Template("""{% load addendum_tags %}{% snippet 'plain' language='es-mx' %}Hello world{% endsnippet %}""")
        c = Context({})
        result = t.render(c)
        self.assertEqual(result, "Hello, humans")

    def test_specified_language(self):
        """Ensure specified language is used, not default"""
        t = Template("""{% load addendum_tags %}{% snippet 'plain' language=lang safe=True %}Hello world{% endsnippet %}""")
        c = Context({'lang': 'en-au'})
        result = t.render(c)
        self.assertEqual(result, "G'day, humans")

    def test_no_specified_language(self):
        """Ensure that the current site language is used"""
        request = RequestFactory().request()
        t = Template("""{% load addendum_tags %}{% snippet 'plain' %}Hello world{% endsnippet %}""")
        c = RequestContext(request, {})
        result = t.render(c)
        self.assertEqual(result, "Hola, humanos")



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SnippetTranslation'
        db.create_table(u'addendum_snippettranslation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('snippet', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', to=orm['addendum.Snippet'])),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=5)),
            ('text', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'addendum', ['SnippetTranslation'])

        # Adding unique constraint on 'SnippetTranslation', fields ['snippet', 'language']
        db.create_unique(u'addendum_snippettranslation', ['snippet_id', 'language'])


    def backwards(self, orm):
        # Removing unique constraint on 'SnippetTranslation', fields ['snippet', 'language']
        db.delete_unique(u'addendum_snippettranslation', ['snippet_id', 'language'])

        # Deleting model 'SnippetTranslation'
        db.delete_table(u'addendum_snippettranslation')


    models = {
        u'addendum.snippet': {
            'Meta': {'ordering': "('key',)", 'object_name': 'Snippet'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '250', 'primary_key': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {})
        },
        u'addendum.snippettranslation': {
            'Meta': {'unique_together': "(('snippet', 'language'),)", 'object_name': 'SnippetTranslation'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'snippet': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': u"orm['addendum.Snippet']"}),
            'text': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['addendum']


"""
Management command to update all snippet cached values.
"""

from django.core.management.base import BaseCommand

from ...models import set_cached_snippet, Snippet


class Command(BaseCommand):
    help = "Updates the cache values for all snippets"

    def handle(self, *args, **kwargs):
        # Already iterating, skip the extra count query
        count = 0
        for snippet in Snippet.objects.all():
            set_cached_snippet(snippet.key)
            count += 1
        self.stdout.write(
                "Refreshed the cache for {0} snippets.".format(count))



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Snippet.key'
        db.alter_column('addendum_snippet', 'key', self.gf('django.db.models.fields.CharField')(max_length=250, primary_key=True))

    def backwards(self, orm):

        # Changing field 'Snippet.key'
        db.alter_column('addendum_snippet', 'key', self.gf('django.db.models.fields.CharField')(max_length=100, primary_key=True))

    models = {
        'addendum.snippet': {
            'Meta': {'ordering': "('key',)", 'object_name': 'Snippet'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '250', 'primary_key': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['addendum']


#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import warnings

from django.conf import settings
from django.core.cache import cache
from django.db import models


def set_cached_snippet(key):
    """
    Adds a dictionary of snippet text and translations to the cache.

    The default text has the key of an empty string.

        {
            "": "Hello, humans",
            "es": "Hola, humanos",
            "en-au": "G'day, humans",
        }

    """
    text_dict = {
        trans.language: trans.text for trans in
        SnippetTranslation.objects.filter(snippet_id=key)
    }
    text_dict.update({'': Snippet.objects.get(key=key).text})
    cache.set('snippet:{0}'.format(key), text_dict)


def get_cached_snippet(key, language=''):
    """
    Fetches the snippet from cache.

    Returns the text value (string) of a Snippet or None.

    This method addes every queried key to the cache to ensure that misses
    doesn't continue to generate database lookups. Since `None` is the
    default return value for a cache miss, the method uses -1 as the miss
    value. If this is returned we know that the value should not be present
    in the database, either.

    :param key: the snippet key (string)
    :param language: optional language code (string)
    :returns: text of snippet (string) or None
    """
    # TODO on fallback try looking for parent language string, e.g. if 'es-ar'
    # is missing then try looking for 'es'.

    snippet = cache.get('snippet:{0}'.format(key))

    # Previous cache miss and DB miss
    if snippet == -1:
        return None

    # First cache miss
    if snippet is None:
        try:
            snippet = Snippet.objects.get(key=key)
        except Snippet.DoesNotExist:
            cache.set('snippet:{0}'.format(key), -1)
            snippet = {'': None}
        else:
            set_cached_snippet(key)

    return snippet.get(language, snippet.get(''))


class CachedManager(models.Manager):

    def get_from_cache(self, key):
        """
        DEPRECATED.

        Use get_cached_snippet instead.
        """
        warnings.warn("The CachedManager is now deprecated, use get_cached_text instead",
                DeprecationWarning)
        snippet = cache.get('snippet:{0}'.format(key))

        if snippet == -1:
            return None

        if snippet is None:
            try:
                snippet = Snippet.objects.get(key=key)
            except Snippet.DoesNotExist:
                cache.set('snippet:{0}'.format(key), -1)
            else:
                cache.set('snippet:{0}'.format(key), snippet)

        return snippet


class Snippet(models.Model):
    """
    Model for storing snippets of text for replacement in templates.

    This should be used for the default language in the case of a multilingual
    app.
    """
    key = models.CharField(max_length=250, primary_key=True)
    text = models.TextField()
    objects = CachedManager()

    class Meta:
        ordering = ('key',)

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        super(Snippet, self).save(*args, **kwargs)
        set_cached_snippet(self.key)
        return self

    def delete(self, **kwargs):
        cache.delete('snippet:{0}'.format(self.key))
        return super(Snippet, self).delete(**kwargs)


class SnippetTranslation(models.Model):
    """
    Additional text copies of the original snippet for use with the specified
    language.
    """
    snippet = models.ForeignKey(Snippet, related_name="translations")
    language = models.CharField(max_length=5, choices=settings.LANGUAGES)
    text = models.TextField()

    class Meta:
        unique_together = ('snippet', 'language')

    def __str__(self):
        return "{0} ({1})".format(self.snippet, self.language)

    def save(self, *args, **kwargs):
        super(SnippetTranslation, self).save(*args, **kwargs)
        set_cached_snippet(self.snippet_id)
        return self

    def delete(self, **kwargs):
        """
        After removing from the database update the snippet cache values.
        """
        return super(Snippet, self).delete(**kwargs)
        set_cached_snippet(self.snippet_id)



from django.contrib import admin
from django.conf import settings

from .models import Snippet, SnippetTranslation


class TranslationAdmin(admin.TabularInline):
    model = SnippetTranslation
    extra = 1


class SnippetAdmin(admin.ModelAdmin):
    list_display = ('key', 'text')
    inlines = [TranslationAdmin]

    def __init__(self, *args, **kwargs):
        """
        Hides the translation inlines if the project does not have i18n
        enabled.
        """
        if not settings.USE_I18N:
            self.inlines = []
        super(SnippetAdmin, self).__init__(*args, **kwargs)


admin.site.register(Snippet, SnippetAdmin)



from django import template
from django.template.base import TemplateSyntaxError
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from ..models import get_cached_snippet


register = template.Library()


def str_bool(val):
    if val.lower() == "true":
        return True
    elif val.lower() == "false":
        return False
    raise ValueError("{0} is not a string representation of a boolean".format(val))


def build_options(bits, tag_name):
    """
    Splits the tag keyword arguments into usable values.
    """

    options = {}

    for bit in bits:
        try:
            option, val = bit.split('=')
        except ValueError:
            raise TemplateSyntaxError("%s has bad or badly formed option arguments." % tag_name)

        if option not in ['safe', 'richtext', 'template', 'language']:
            raise TemplateSyntaxError("%s recieved an invalid option." % tag_name)

        # Backwards compatibility
        if option == 'richtext':
            option = 'safe'

        options.update({option: val})

    return options


@register.tag
def snippet(parser, token):
    """
    Demarcates replaceable text, outputs the wrapped text or the text of a
    snippet with matching key.

    Examples::

        {% snippet 'greeting' %}Hello world{% endsnippet %}

        {% snippet 'greeting' richtext=True %}<p>Hey!</p>{% endsnippet %}
    """
    nodelist = parser.parse(('endsnippet',))
    parser.delete_first_token()

    try:
        tag_name = token.split_contents()
        bits = token.split_contents()[1:]
    except IndexError:
        raise TemplateSyntaxError("%s tag takes at least one argument" % bits[0])

    key = bits[0]
    options = build_options(bits[1:], tag_name)

    return SnippetNode(nodelist, key, **options)


class SnippetNode(template.Node):

    safe = False
    template = False
    language = ''

    def __init__(self, nodelist, key, **options):
        self.nodelist = nodelist
        self.key = template.Variable(key)
        for k, v in options.items():
            setattr(self, k, template.Variable(v))

    def render(self, context):

        # Handle key as context variable or key as string
        try:
            key = self.key.resolve(context)
        except AttributeError:
            key = self.key[1:-1]

        if self.language != '':
            try:
                language = self.language.resolve(context)
            except AttributeError:
                language = self.language[1:-1]
        else:
            language = context.get('LANGUAGE_CODE', self.language)

        snippet = get_cached_snippet(key, language)

        if snippet is None:
            output = self.nodelist.render(context)
            return output

        if self.template:
            if self.safe:
                context.autoescape = False
                return mark_safe(template.Template(snippet).render(context))
            return template.Template(snippet).render(context)

        if self.safe:
            return mark_safe(snippet)

        return conditional_escape(snippet)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Snippet',
            fields=[
                ('key', models.CharField(max_length=250, serialize=False, primary_key=True)),
                ('text', models.TextField()),
            ],
            options={
                'ordering': ('key',),
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.views.generic import TemplateView

admin.autodiscover()


urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^$', view=TemplateView.as_view(template_name='index.html'),
        name='index')
)



# -*- coding: utf-8 -*-

import os
import sys
import addendum

from setuptools import setup, find_packages


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()


with open('README.rst') as f:
    readme = f.read()


setup(
    name='django-addendum',
    version=addendum.__version__,
    description='Simple template-based content swapping for CMS-less sites',
    long_description=readme,
    author='Ben Lopatin',
    author_email='ben.lopatin@wellfireinteractive.com',
    url='https://github.com/wellfire/django-addendum',
    license='BSD License',
    packages=find_packages(exclude=('example', 'docs')),
    platforms=['OS Independent'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
    ],
    include_package_data=True,
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Snippet'
        db.create_table(u'addendum_snippet', (
            ('key', self.gf('django.db.models.fields.CharField')(max_length=100, primary_key=True)),
            ('text', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'addendum', ['Snippet'])


    def backwards(self, orm):
        # Deleting model 'Snippet'
        db.delete_table(u'addendum_snippet')


    models = {
        u'addendum.snippet': {
            'Meta': {'ordering': "('key',)", 'object_name': 'Snippet'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '100', 'primary_key': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['addendum']